#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <hosties3>
#include <regex>
#include <smlib>

#include "hosties3/globals.sp"

#include "hosties3/colors.sp"
#include "hosties3/morecolors.sp"

#include "hosties3/native.sp"

#include "hosties3/chat.sp"
#include "hosties3/client.sp"
#include "hosties3/misc.sp"

#define PLUGIN_NAME HOSTIES3_NAME ... "Core"
#define PLUGIN_CONFIG HOSTIES3_CONFIG ... "core.cfg"

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = HOSTIES3_AUTHOR,
	version = HOSTIES3_VERSION,
	description = "Rewrite of SM_Hosties v2",
	url = HOSTIES3_URL
};

public APLRes:AskPluginLoad2(Handle myself, bool late, char[] error, int err_max)
{
	Native_AskPluginLoad2();

	RegPluginLibrary("hosties3");

	return APLRes_Success;
}

public OnPluginStart()
{
	GetGame();
	Hosties3_CheckGame();

	RegAdminCmd("sm_reloadhconfig", Command_ReloadHConfig, ADMFLAG_ROOT);

	HookEvent("round_start", Event_OnRoundStart);
	HookEvent("round_end", Event_OnRoundEnd);
	HookEvent("player_spawn", Event_OnPlayerSpawn);
	HookEvent("player_hurt", Event_OnPlayerHurt);
	HookEvent("player_death", Event_OnPlayerDeath);

	LoadConfig();
}

public OnAllPluginsLoaded()
{
	Call_StartForward(g_hOnPluginLoaded);
	Call_Finish();
}

public OnMapStart()
{
	Call_StartForward(g_hOnMapStart);
	Call_Finish();
}

public OnClientPostAdminCheck(int client)
{
	CreateTimer(3.0, Timer_OnClientReady, client);
}

public Action:Command_ReloadHConfig(int client, args)
{
	LoadConfig();
}

public Action:Event_OnRoundStart(Handle event, const char[] name, bool dontbroadcast)
{
	Call_StartForward(g_hOnRoundStart);
	Call_Finish();
}

public OnClientDisconnect(int client)
{
	if(Hosties3_IsClientValid(client))
	{
		Call_StartForward(g_hOnClientDisconnect);
		Call_PushCell(client);
		Call_Finish();
	}
}

public Event_OnRoundEnd(Handle event, const char[] name, bool dontbroadcast)
{
	Call_StartForward(g_hOnRoundEnd);
	Call_PushCell(GetEventInt(event, "winner"));
	Call_Finish();
}

public Action:Event_OnPlayerSpawn(Handle event, const char[] name, bool dontbroadcast)
{
	int client = GetClientOfUserId(GetEventInt(event, "userid"));

	RequestFrame(OnPlayerSpawn, client);
}

public OnPlayerSpawn(any:client)
{
	if(Hosties3_IsClientValid(client))
	{
		Call_StartForward(g_hOnPlayerSpawn);
		Call_PushCell(client);
		Call_Finish();
	}
}

public Action:Event_OnPlayerHurt(Handle event, const char[] name, bool dontbroadcast)
{
	int victim = GetClientOfUserId(GetEventInt(event, "userid"));
	int attacker = GetClientOfUserId(GetEventInt(event, "attacker"));
	char weapon[32];

	if(Hosties3_IsClientValid(victim) && Hosties3_IsClientValid(attacker))
	{
		Call_StartForward(g_hOnPlayerHurt);
		Call_PushCell(victim);
		Call_PushCell(attacker);
		Call_PushString(weapon);
		Call_Finish();
	}
}

public Action:Event_OnPlayerDeath(Handle event, const char[] name, bool dontbroadcast)
{
	int victim = GetClientOfUserId(GetEventInt(event, "userid"));
	int attacker = GetClientOfUserId(GetEventInt(event, "attacker"));
	int assister = GetClientOfUserId(GetEventInt(event, "assister"));
	bool headshot = GetEventBool(event, "headshot");
	char weapon[32];

	if(Hosties3_IsClientValid(victim) && Hosties3_IsClientValid(attacker))
	{
		Call_StartForward(g_hOnPlayerDeath);
		Call_PushCell(victim);
		Call_PushCell(attacker);
		Call_PushCell(assister);
		Call_PushString(weapon);
		Call_PushCell(headshot);
		Call_Finish();
	}
}

public Action:Timer_OnClientReady(Handle timer, any:client)
{
	if(Hosties3_IsClientValid(client))
	{
		Call_StartForward(g_hOnClientReady);
		Call_PushCell(client);
		Call_Finish();
	}
}

GetGame()
{
	if (GetEngineVersion() == Engine_CSS)
	{
		g_iGame = Game_CSS;
	}
	else if (GetEngineVersion() == Engine_CSGO)
	{
		g_iGame = Game_CSGO;
	}
	else
	{
		g_iGame = Game_Unsupported;
	}
}

LoadConfig()
{
	Handle hConfig = CreateKeyValues("Hosties3");

	if(!FileExists(PLUGIN_CONFIG))
	{
		SetFailState("[Hosties3] 'addons/sourcemod/configs/hosties3/core.cfg' not found!");
		return;
	}

	FileToKeyValues(hConfig, PLUGIN_CONFIG);
	if (KvJumpToKey(hConfig, "Core"))
	{
		g_iEnable = KvGetNum(hConfig, "Enable", 1);
		g_iAutoUpdate = KvGetNum(hConfig, "AutoUpdate", 1);
		g_iLogLevel = KvGetNum(hConfig, "LogLevel", 3);
		KvGetString(hConfig, "Tag", g_sTag, sizeof(g_sTag));
	}
	else
	{
		SetFailState("Config for 'Hosties3' not found!");
		return;
	}

	if(g_iLogLevel <= 2)
	{
		Hosties3_LogFile(HOSTIES3_PATH, "core", _, DEBUG, "[Hosties3] Enable: %d", g_iEnable);
		Hosties3_LogFile(HOSTIES3_PATH, "core", _, DEBUG, "[Hosties3] AutoUpdate: %d", g_iAutoUpdate);
		Hosties3_LogFile(HOSTIES3_PATH, "core", _, DEBUG, "[Hosties3] LogLevel: %d", g_iLogLevel);
		Hosties3_LogFile(HOSTIES3_PATH, "core", _, DEBUG, "[Hosties3] Tag: %s", g_sTag);
	}
	if(g_iEnable != 1)
	{
		SetFailState("'Hosties3' is deactivated!");
		return;
	}

	CloseHandle(hConfig);

	Call_StartForward(g_hOnConfigsLoaded);
	Call_Finish();
}
