#pragma semicolon 1

#include <sourcemod>
#include <cstrike>
#include <smlib>
#include <hosties3>

#define PLUGIN_NAME HOSTIES3_NAME ... "Gunplant Prevention"
#define PLUGIN_CONFIG HOSTIES3_CONFIG ... "gunplantprevention.cfg"

enum GunplantPrevention:
{
	DISABLED = 0,
	ENABLED_FULL,
	ENABLED_ALLOWEMPTY,
};

enum GunplantPunishment:
{
	DEATH = 0,
	REMOVEWEAPON,
	DEATH_REMOVEWEAPON,
};

int g_iEnable;
int g_iLogLevel;
int g_iPunishment;

float g_fGunplantPreventionTime;

char g_sTag[64];

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = HOSTIES3_AUTHOR,
	version = HOSTIES3_VERSION,
	description = "Rewrite of SM_Hosties v2",
	url = "www.last-resistance.co.uk"
};

public Hosties3_OnPluginLoaded()
{
	Hosties3_IsLoaded();
	LoadConfig();
}

public Hosties3_OnConfigsLoaded()
{
	LoadConfig();
}

public Action:CS_OnCSWeaponDrop(int client, int iWeapon)
{
	//TODO: make sure it doesn't trigger during the player's last request
	if (g_iEnable && Hosties3_IsClientValid(client, true) && GetClientTeam(client) == CS_TEAM_CT)
	{
		if (GunplantPrevention:g_iEnable == ENABLED_ALLOWEMPTY)
		{
			int iPrimaryAmmo;
			char sWeaponName[32];
			GetEntityClassname(iWeapon, sWeaponName, sizeof(sWeaponName));
			Client_GetWeaponPlayerAmmo(client, sWeaponName, iPrimaryAmmo);
			int iSecondaryAmmo = Weapon_GetPrimaryClip(iWeapon);
			
			if (iPrimaryAmmo == 0 && iSecondaryAmmo == 0)
			{
				return Plugin_Continue;
			}
		}
		
		Handle hData = CreateDataPack();
		WritePackCell(hData, client);
		WritePackCell(hData, iWeapon);
		
		CreateTimer(g_fGunplantPreventionTime, Timer_GunPlantPrevention, hData, TIMER_FLAG_NO_MAPCHANGE);
	}
	return Plugin_Continue;
}

public Action:Timer_GunPlantPrevention(Handle hTimer, Handle hData)
{
	ResetPack(hData);
	int iOriginalOwner = ReadPackCell(hData);
	int iWeapon = ReadPackCell(hData);
	
	if (!IsValidEdict(iWeapon) || !Hosties3_IsClientValid(iOriginalOwner, true))
	{
		return Plugin_Stop;
	}

	char sWeaponName[32];
	GetEntityClassname(iWeapon, sWeaponName, sizeof(sWeaponName));
	
	if (!strncmp(sWeaponName, "weapon_", 7))
	{
		return Plugin_Stop;
	}
	
	int iNewOwner = GetEntPropEnt(iWeapon, Prop_Data, "m_hOwnerEntity");
	if (iNewOwner == -1)
	{
		return Plugin_Stop;
	}
	
	if (Hosties3_IsClientValid(iNewOwner, true) && GetClientTeam(iNewOwner) != GetClientTeam(iOriginalOwner))
	{
		switch(g_iPunishment)
		{
			case DEATH:
			{
				ForcePlayerSuicide(iOriginalOwner);
			}
			case REMOVEWEAPON:
			{
				if (Client_GetActiveWeapon(iNewOwner) == iWeapon)
				{
					FakeClientCommand(iNewOwner, "use weapon_knife");
				}
				AcceptEntityInput(iWeapon, "kill");
			}
			case DEATH_REMOVEWEAPON:
			{
				if (Client_GetActiveWeapon(iNewOwner) == iWeapon)
				{
					FakeClientCommand(iNewOwner, "use weapon_knife");
				}
				AcceptEntityInput(iWeapon, "kill");
				ForcePlayerSuicide(iOriginalOwner);
			}
		}
		
		if(g_iLogLevel <= 2)
		{
			Hosties3_LogFile(HOSTIES3_PATH, "gunplantprevention", _, DEBUG, "[Gunplant Prevention] \"%L\" tried to gunplant \"%L\"!", iOriginalOwner, iNewOwner);
		}
	}
	
	return Plugin_Stop;
}

LoadConfig()
{
	Handle hConfig = CreateKeyValues("Hosties3");

	if(!FileExists(PLUGIN_CONFIG))
	{
		SetFailState("[Hosties3] 'addons/sourcemod/configs/hosties3/gunplantprevention.cfg' not found!");
		return;
	}

	FileToKeyValues(hConfig, PLUGIN_CONFIG);
	if (KvJumpToKey(hConfig, "GunplantPrevention"))
	{
		g_iEnable = KvGetNum(hConfig, "Enable", 1);
		g_iPunishment = KvGetNum(hConfig, "Punishment", 1);
		g_fGunplantPreventionTime = KvGetFloat(hConfig, "PreventionTime", 1.337);
		g_iLogLevel = Hosties3_GetLogLevel();
		Hosties3_GetTag(g_sTag, sizeof(g_sTag));
	}
	else
	{
		SetFailState("Config for 'Gunplant Prevention' not found!");
		return;
	}

	if(g_iLogLevel <= 2)
	{
		Hosties3_LogFile(HOSTIES3_PATH, "gunplantprevention", _, DEBUG, "[Gunplant Prevention] Enable: %d", g_iEnable);
	}

	if(g_iEnable < 1 || g_iEnable > 2)
	{
		SetFailState("Gunplant Prevention is deactivated!");
		return;
	}

	CloseHandle(hConfig);
}