#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <hosties3>
#include <cstrike>
#include <multicolors>
#include <thelpers/thelpers>

#define PLUGIN_NAME HOSTIES3_NAME ... "NoBlock"
#define PLUGIN_CONFIG HOSTIES3_CONFIG ... "noblock.cfg"

int g_iOffsCollision = -1;
int g_iEnable;
int g_iLogLevel;
int g_iSpawnMode;
int g_iTime;
int g_iCommands;

char g_sTag[64];
char g_sNoBlockComList[8][32];
char g_sNoBlockCom[128];

Handle g_hTimer[MAXPLAYERS + 1] = {INVALID_HANDLE, ...};

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = HOSTIES3_AUTHOR,
	version = HOSTIES3_VERSION,
	description = "Rewrite of SM_Hosties v2",
	url = HOSTIES3_URL
};

public OnPluginStart()
{
	Hosties3_IsLoaded();
	Hosties3_CheckGame();

	LoadTranslations("common.phrases.txt");
	// LoadTranslations("hosties3_noblock.phrases");

	LoadConfig();

	g_iOffsCollision = FindSendPropOffs("CBaseEntity", "m_CollisionGroup");
	if (g_iOffsCollision == -1)
	{
		SetFailState("Failed to get offset for CBaseEntity::m_CollisionGroup");
	}
}

public Hosties3_OnConfigsLoaded()
{
	LoadConfig();
}

public Action Command_Block(int client, args)
{
	if(!Hosties3_IsClientValid(client))
	{
		return;
	}

	if(!IsPlayerAlive(client))
	{
		return;
	}

	ShowMenu(client);
}

public OnClientDisconnect(int client)
{
	if(g_hTimer[client] != INVALID_HANDLE)
	{
		CloseHandle(g_hTimer[client]);
		g_hTimer[client] = INVALID_HANDLE;
	}
}

public Hosties3_OnPlayerSpawn(int client)
{
	if(Hosties3_IsClientValid(client))
	{
		RequestFrame(OnPlayerSpawn, client);
	}
}

public Hosties3_OnPlayerDeath(int victim, int attacker, int assister, const char[] weapon, bool headshot)
{
	if(g_hTimer[victim] != INVALID_HANDLE)
	{
		CloseHandle(g_hTimer[victim]);
		g_hTimer[victim] = INVALID_HANDLE;
	}
}

ShowMenu(client)
{
	char sTitle[128], sTime[128], sActivate[64], sDeactivate[64];

	Format(sTitle, sizeof(sTitle), "%T", "NoBlock Menu Title", client);
	Format(sActivate, sizeof(sActivate), "%T", "NoBlock Menu Activate", client);
	Format(sDeactivate, sizeof(sDeactivate), "%T", "NoBlock Menu Deactivate", client);
	Format(sTime, sizeof(sTime), "%T", "NoBlock Menu Time", client, g_iTime);

	Menu menu = CreateMenu(Menu_Block);
	menu.SetTitle(sTitle);

	if(g_hTimer[client] == INVALID_HANDLE)
	{
		if(IsClientBlock(client))
		{
			menu.AddItem("", sActivate, ITEMDRAW_DISABLED);
			menu.AddItem("", sTime, ITEMDRAW_DISABLED);
			menu.AddItem("off", sDeactivate);
		}
		else
		{
			menu.AddItem("on", sActivate);
			menu.AddItem("time", sTime);
			menu.AddItem("", sDeactivate, ITEMDRAW_DISABLED);
		}
	}
	else
	{
		menu.AddItem("", "Es läuft bereits ein Timer", ITEMDRAW_DISABLED);
	}

	menu.ExitButton = true;
	menu.Display(client, MENU_TIME_FOREVER);
}

public Menu_Block(Menu menu, MenuAction action, client, param)
{
	if(action == MenuAction_Select)
	{
		CBasePlayer player = CBasePlayer(client);
		char sParam[256];
		menu.GetItem(param, sParam, sizeof(sParam));

		if(StrEqual(sParam, "on"))
		{
			player.SetBlock(g_iOffsCollision, true);
			// Todo... Add Translations
			Hosties3_PrintToChat(client, "%T", "NoBlock On", client);
			ShowMenu(client);
		}
		else if(StrEqual(sParam, "time"))
		{
			if(!IsClientStuck(client))
			{
				player.SetBlock(g_iOffsCollision, false);
				Hosties3_PrintToChat(client, "%T", "NoBlock On", client);
				g_hTimer[client] = CreateTimer(float(g_iTime), Timer_Block, client);
				ShowMenu(client);
			}
			else
			{
				RequestFrame(CheckClientTime, client);
			}
		}
		else if(StrEqual(sParam, "off"))
		{
			if(!IsClientStuck(client))
			{
				player.SetBlock(g_iOffsCollision, false);
				// Todo... Add Translations
				Hosties3_PrintToChat(client, "%T", "NoBlock Off", client);
				ShowMenu(client);
			}
			else
			{
				RequestFrame(CheckClient, client);
			}
		}
	}
}

public CheckClient(any client)
{
	if(Hosties3_IsClientValid(client) && IsPlayerAlive(client))
	{
		CBasePlayer player = CBasePlayer(client);

		if(!IsClientStuck(client))
		{
			player.SetBlock(g_iOffsCollision, false);
			Hosties3_PrintToChat(client, "%T", "NoBlock Off", client);
			ShowMenu(client);
		}
		else
		{
			RequestFrame(CheckClient, client);
		}
	}
}

public CheckClientTime(any client)
{
	if(Hosties3_IsClientValid(client) && IsPlayerAlive(client))
	{
		CBasePlayer player = CBasePlayer(client);

		if(!IsClientStuck(client))
		{
			player.SetBlock(g_iOffsCollision, true);
			Hosties3_PrintToChat(client, "%T", "NoBlock On", client);
			g_hTimer[client] = CreateTimer(float(g_iTime), Timer_Block, client);
			ShowMenu(client);
		}
		else
		{
			RequestFrame(CheckClientTime, client);
		}
	}
}

public OnPlayerSpawn(any client)
{
	if(g_iSpawnMode && !IsClientBlock(client))
	{
		CBasePlayer player = CBasePlayer(client);
		player.SetBlock(g_iOffsCollision, true);
	}
}

public Action Timer_Block(Handle timer, any client)
{
	CBasePlayer player = CBasePlayer(client);

	player.SetBlock(g_iOffsCollision, false);
	// Todo... Add Translations
	Hosties3_PrintToChat(client, "%T", "NoBlock Off", client);
	g_hTimer[client] = INVALID_HANDLE;
	ShowMenu(client);
	return Plugin_Stop;
}

stock bool IsClientStuck(client)
{
	float vOrigin[3];
	float vMins[3];
	float vMaxs[3];

	GetClientAbsOrigin(client, vOrigin);

	GetEntPropVector(client, Prop_Send, "m_vecMins", vMins);
	GetEntPropVector(client, Prop_Send, "m_vecMaxs", vMaxs);
	
	TR_TraceHullFilter(vOrigin, vOrigin, vMins, vMaxs, MASK_ALL, FilterOnlyPlayers, client);

	return TR_DidHit();
}

public bool IsClientBlock(client)
{
	if(GetEntData(client, g_iOffsCollision, 4) == 5)
	{
		return false;
	}
	return true;
}

public bool FilterOnlyPlayers(client, contentsMask, any data)
{
	if(client != data && client > 0 && client <= MaxClients)
	{
    	return true;
	}
	return false;
}

LoadConfig()
{
	Handle hConfig = CreateKeyValues("Hosties3");

	if(!FileExists(PLUGIN_CONFIG))
	{
		SetFailState("[Hosties3] 'addons/sourcemod/configs/hosties3/noblock.cfg' not found!");
		return;
	}

	FileToKeyValues(hConfig, PLUGIN_CONFIG);
	if (KvJumpToKey(hConfig, "NoBlock"))
	{
		g_iEnable = KvGetNum(hConfig, "Enable", 1);
		g_iSpawnMode = KvGetNum(hConfig, "NoBlockMode", 10);
		g_iTime = KvGetNum(hConfig, "TimeNoBlock", 10);
		KvGetString(hConfig, "Commands", g_sNoBlockCom, sizeof(g_sNoBlockCom), "b;block;ub;unblock;nb;noblock;");
		g_iLogLevel = Hosties3_GetLogLevel();
		Hosties3_GetTag(g_sTag, sizeof(g_sTag));
	}
	else
	{
		SetFailState("Config for 'NoBlock' not found!");
		return;
	}

	if(g_iLogLevel <= 2)
	{
		Hosties3_LogFile(HOSTIES3_PATH, "noblock", _, DEBUG, "[NoBlock] Enable: %d", g_iEnable);
		Hosties3_LogFile(HOSTIES3_PATH, "noblock", _, DEBUG, "[NoBlock] Mode: %d", g_iSpawnMode);
		Hosties3_LogFile(HOSTIES3_PATH, "noblock", _, DEBUG, "[NoBlock] NoBlock Time: %d", g_iTime);
		Hosties3_LogFile(HOSTIES3_PATH, "noblock", _, DEBUG, "[NoBlock] NoBlock Commands: %s", g_sNoBlockCom);
	}

	g_iCommands = ExplodeString(g_sNoBlockCom, ";", g_sNoBlockComList, sizeof(g_sNoBlockComList), sizeof(g_sNoBlockComList[]));

	for(int i = 0; i < g_iCommands; i++)
	{
			char sBuffer[32];
			Format(sBuffer, sizeof(sBuffer), "sm_%s", g_sNoBlockComList[i]);
			RegConsoleCmd(sBuffer, Command_Block);
			Hosties3_LogFile(HOSTIES3_PATH, "noblock", _, DEBUG, "[NoBlock] Register Command: %s Full: %s", g_sNoBlockComList[i], sBuffer);
	}

	if(!g_iEnable)
	{
		SetFailState("[Hosties3] NoBlock Feature is disabled!");
		return;
	}

	CloseHandle(hConfig);
}
