#pragma semicolon 1

#include <sourcemod>
#include <cstrike>
#include <hosties3>

#define PLUGIN_NAME HOSTIES3_NAME ... "Respawn"
#define PLUGIN_CONFIG HOSTIES3_CONFIG ... "respawn.cfg"

int g_iEnable;
int g_iMessageMode;
int g_iLogLevel;

char g_sTag[64];

int g_iRespawnCom;
char g_sRespawnComList[8][32];
char g_sRespawnCom[128];

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = HOSTIES3_AUTHOR,
	version = HOSTIES3_VERSION,
	description = "Rewrite of SM_Hosties v2",
	url = HOSTIES3_URL
};

public Hosties3_OnPluginLoaded()
{
	Hosties3_IsLoaded();
	Hosties3_CheckGame();

	LoadTranslations("common.phrases.txt");
	LoadTranslations("hosties3_respawn.phrases");

	LoadConfig();
}

public Hosties3_OnConfigsLoaded()
{
	LoadConfig();
}

public Action:Command_Respawn(int client, args)
{
	if(!g_iEnable)
	{
		Hosties3_ReplyToCommand(client, "%T", "CommandDisabled", client, g_sTag);
		return Plugin_Handled;
	}

	if (args != 1)
	{
		Hosties3_ReplyToCommand(client, "%T", "InvalidParameter", client, g_sTag);
		return Plugin_Handled;
	}
	
	char sArg[65];
	GetCmdArg(1, sArg, sizeof(sArg));

	char target_name[MAX_TARGET_LENGTH];
	int target_list[MAXPLAYERS];
	int target_count;
	bool tn_is_ml;
	
	if ((target_count = ProcessTargetString(sArg, client, target_list, MAXPLAYERS, COMMAND_FILTER_DEAD, target_name, sizeof(target_name), tn_is_ml)) <= 0)
	{
		ReplyToTargetError(client, target_count);
		return Plugin_Handled;
	}

	for (int i = 0; i < target_count; i++)
	{
		int target = target_list[i];

		if(!Hosties3_IsClientValid(target))
		{
			Hosties3_ReplyToCommand(client, "%T", "TargetInvalid", client, g_sTag);
			return Plugin_Handled;
		}

		if (GetClientTeam(target) != CS_TEAM_CT && GetClientTeam(target) != CS_TEAM_T)
		{
			Hosties3_ReplyToCommand(client, "TargetInvalidTeam", client, g_sTag);
			return Plugin_Handled;
		}

		CS_RespawnPlayer(target);
		Hosties3_LogFile(HOSTIES3_PATH, "rules", _, INFO, "\"%L\" was respawned by \"%L\"!", target, client);

		if(g_iMessageMode)
		{
			Hosties3_LoopClients(j)
			{
				if(Hosties3_IsClientValid(j))
				{
					Hosties3_PrintToChat(j, "%T", "PlayerRespawned", j, g_sTag, target, client);
				}
			}
		}
		else if (g_iMessageMode == 2)
		{
			Hosties3_LoopClients(j)
			{
				if(Hosties3_IsClientValid(j, _, _, true))
				{
					Hosties3_PrintToChat(j, "%T", "PlayerRespawned", j, g_sTag, target, client);
				}
			}
		}
	}
	return Plugin_Continue;
}

LoadConfig()
{
	Handle hConfig = CreateKeyValues("Hosties3");

	if(!FileExists(PLUGIN_CONFIG))
	{
		SetFailState("[Hosties3] 'addons/sourcemod/configs/hosties3/respawn.cfg' not found!");
		return;
	}

	FileToKeyValues(hConfig, PLUGIN_CONFIG);
	if (KvJumpToKey(hConfig, "Respawn"))
	{
		g_iEnable = KvGetNum(hConfig, "Enable", 1);
		g_iMessageMode = KvGetNum(hConfig, "MessageMode", 1);
		g_iLogLevel = Hosties3_GetLogLevel();

		KvGetString(hConfig, "RespawnCommands", g_sRespawnCom, sizeof(g_sRespawnCom), "respawn;1up");
		Hosties3_GetTag(g_sTag, sizeof(g_sTag));
	}
	else
	{
		SetFailState("Config for 'Respawn' not found!");
		return;
	}

	if(g_iLogLevel <= 2)
	{
		Hosties3_LogFile(HOSTIES3_PATH, "respawn", _, DEBUG, "[Respawn] Enable: %d", g_iEnable);
		Hosties3_LogFile(HOSTIES3_PATH, "respawn", _, DEBUG, "[Respawn] Message Mode: %d", g_iMessageMode);
		Hosties3_LogFile(HOSTIES3_PATH, "respawn", _, DEBUG, "[Respawn] Respawn Commands: %s", g_sRespawnCom);
	}

	g_iRespawnCom = ExplodeString(g_sRespawnCom, ";", g_sRespawnComList, sizeof(g_sRespawnComList), sizeof(g_sRespawnComList[]));

	for(int i = 0; i < g_iRespawnCom; i++)
	{
		char sBuffer[32];
		Format(sBuffer, sizeof(sBuffer), "sm_%s", g_sRespawnComList[i]);
		RegAdminCmd(sBuffer, Command_Respawn, ADMFLAG_GENERIC);
		Hosties3_LogFile(HOSTIES3_PATH, "respawn", _, DEBUG, "[Respawn] Register Command: %s Full: %s", g_sRespawnComList[i], sBuffer);
	}

	if(!g_iEnable)
	{
		SetFailState("[Hosties3] Respawn Feature is disabled!");
		return;
	}

	CloseHandle(hConfig);
}
