#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <cstrike>
#include <smlib>
#include <hosties3>

#define PLUGIN_NAME HOSTIES3_NAME ... "Weapons"
#define PLUGIN_CONFIG HOSTIES3_CONFIG ... "weapons.cfg"

int g_iEnable;
int g_iLogLevel;
int g_iChatMessage;

int g_iCTWeaponList;
char g_sCTWeapons[256];
char g_sCTWeaponsList[8][32];

int g_iTWeaponList;
char g_sTWeapons[256];
char g_sTWeaponsList[8][32];

char g_sTag[64];

Handle g_hAmmoList;

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = HOSTIES3_AUTHOR,
	version = HOSTIES3_VERSION,
	description = "Rewrite of SM_Hosties v2",
	url = HOSTIES3_URL
};

public Hosties3_OnPluginLoaded()
{
	Hosties3_IsLoaded();
	Hosties3_CheckGame();

	LoadTranslations("hosties3_weapons.phrases");

	LoadConfig();
}

public Hosties3_OnConfigsLoaded()
{
	LoadConfig();
}

public Hosties3_OnPlayerSpawn(int client)
{
	Hosties3_StripClientAll(client, true);

	g_iCTWeaponList = ExplodeString(g_sCTWeapons, ";", g_sCTWeaponsList, sizeof(g_sCTWeaponsList), sizeof(g_sCTWeaponsList[]));
	g_iTWeaponList = ExplodeString(g_sTWeapons, ";", g_sTWeaponsList, sizeof(g_sTWeaponsList), sizeof(g_sTWeaponsList[]));

	if(!IsFakeClient(client) && GetClientTeam(client) == CS_TEAM_T)
	{
		for(int i = 0; i < g_iTWeaponList; i++)
		{
			char sBuffer[32];
			Format(sBuffer, sizeof(sBuffer), "weapon_%s", g_sTWeaponsList[i]);

			if(!StrEqual(sBuffer, "weapon_", false))
			{
				int iWeapon = GivePlayerItem(client, sBuffer);
				EquipPlayerWeapon(client, iWeapon);
				SetAmmo(client, iWeapon);

				if(g_iChatMessage)
				{
					Hosties3_PrintToChat(client, "%T", "GetWeapon", client, g_sTag, g_sTWeaponsList[i]);
				}

				if(g_iLogLevel <= 2)
				{
					Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] Player: \"%L\" - Team: %d - Weapon: %s", client, GetClientTeam(client), g_sTWeaponsList[i]);
				}
			}
		}
	}

	if(!IsFakeClient(client) && GetClientTeam(client) == CS_TEAM_CT)
	{
		for(int i = 0; i < g_iCTWeaponList; i++)
		{
			char sBuffer[32];
			Format(sBuffer, sizeof(sBuffer), "weapon_%s", g_sCTWeaponsList[i]);

			if(!StrEqual(sBuffer, "weapon_", false))
			{
				int iWeapon = GivePlayerItem(client, sBuffer);
				EquipPlayerWeapon(client, iWeapon);
				SetAmmo(client, iWeapon);

				if(g_iChatMessage)
				{
					Hosties3_PrintToChat(client, "%T", "GetWeapon", client, g_sTag, g_sCTWeaponsList[i]);
				}

				if(g_iLogLevel <= 2)
				{
					Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] Player: \"%L\" - Team: %d - Weapon: %s", client, GetClientTeam(client), g_sCTWeaponsList[i]);
				}
			}
		}
	}
}

SetAmmo(client, weapon)
{
	int iOld, iNew;
	char sClass[64];

	iOld = Weapon_GetPrimaryClip(weapon);
	
	GetEntityClassname(weapon, sClass, sizeof(sClass));
	
	if(!GetTrieValue(g_hAmmoList, sClass, iNew))
	{
		iNew = iOld * 3;
	}
	
	Client_SetWeaponPlayerAmmoEx(client, weapon, iNew, -1);
}

LoadConfig()
{
	Handle hConfig = CreateKeyValues("Hosties3");

	if(!FileExists(PLUGIN_CONFIG))
	{
		SetFailState("[Hosties3] 'addons/sourcemod/configs/hosties3/weapons.cfg' not found!");
		return;
	}

	FileToKeyValues(hConfig, PLUGIN_CONFIG);
	if (KvJumpToKey(hConfig, "Weapons"))
	{
		g_iEnable = KvGetNum(hConfig, "Enable", 1);
		g_iChatMessage = KvGetNum(hConfig, "ChatMessage", 1);
		g_iLogLevel = Hosties3_GetLogLevel();
		
		KvGetString(hConfig, "CT", g_sCTWeapons, sizeof(g_sCTWeapons));
		KvGetString(hConfig, "T", g_sTWeapons, sizeof(g_sTWeapons));
		
		Hosties3_GetTag(g_sTag, sizeof(g_sTag));
	}
	else
	{
		SetFailState("Config for 'Weapons' not found!");
		return;
	}

	if(g_iLogLevel <= 2)
	{
		if(StrEqual(g_sTWeapons, "", false))
		{
			Format(g_sTWeapons, sizeof(g_sTWeapons), "no weapons for ts");
		}

		Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] Enable: %d", g_iEnable);
		Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] ChatMessage: %d", g_iChatMessage);
		Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] CT Weapons: %s", g_sCTWeapons);
		Hosties3_LogFile(HOSTIES3_PATH, "weapons", _, DEBUG, "[Weapons] T Weapons: %s", g_sTWeapons);
	}

	if(g_iEnable != 1)
	{
		SetFailState("'Weapons' is deactivated!");
		return;
	}

	g_hAmmoList = CreateTrie();

	SetTrieValue(g_hAmmoList, "weapon_ak47", 90);
	SetTrieValue(g_hAmmoList, "weapon_awp", 30);
	SetTrieValue(g_hAmmoList, "weapon_aug", 90);
	SetTrieValue(g_hAmmoList, "weapon_deagle", 35);
	SetTrieValue(g_hAmmoList, "weapon_elite", 120);
	SetTrieValue(g_hAmmoList, "weapon_famas", 90);
	SetTrieValue(g_hAmmoList, "weapon_fiveseven", 100);
	SetTrieValue(g_hAmmoList, "weapon_galil", 90);
	SetTrieValue(g_hAmmoList, "weapon_g3sg1", 90);
	SetTrieValue(g_hAmmoList, "weapon_glock", 120);
	SetTrieValue(g_hAmmoList, "weapon_m4a1", 90);
	SetTrieValue(g_hAmmoList, "weapon_m3", 32);
	SetTrieValue(g_hAmmoList, "weapon_mac10", 100);
	SetTrieValue(g_hAmmoList, "weapon_mp5navy", 120);
	SetTrieValue(g_hAmmoList, "weapon_m249", 200);
	SetTrieValue(g_hAmmoList, "weapon_p228", 52);
	SetTrieValue(g_hAmmoList, "weapon_p90", 100);
	SetTrieValue(g_hAmmoList, "weapon_scout", 90);
	SetTrieValue(g_hAmmoList, "weapon_sg552", 90);
	SetTrieValue(g_hAmmoList, "weapon_sg550", 90);
	SetTrieValue(g_hAmmoList, "weapon_tmp", 120);
	SetTrieValue(g_hAmmoList, "weapon_ump45", 100);
	SetTrieValue(g_hAmmoList, "weapon_usp", 100);
	SetTrieValue(g_hAmmoList, "weapon_xm1014",32);
	SetTrieValue(g_hAmmoList, "weapon_hkp2000", 52);
	SetTrieValue(g_hAmmoList, "weapon_p250", 52);
	SetTrieValue(g_hAmmoList, "weapon_tec9", 120);
	SetTrieValue(g_hAmmoList, "weapon_ssg08", 90);
	SetTrieValue(g_hAmmoList, "weapon_sg556", 90);
	SetTrieValue(g_hAmmoList, "weapon_sg553", 90);
	SetTrieValue(g_hAmmoList, "weapon_galilar", 90);
	SetTrieValue(g_hAmmoList, "weapon_scar20", 90);
	SetTrieValue(g_hAmmoList, "weapon_mp7", 120);
	SetTrieValue(g_hAmmoList, "weapon_nova", 32);
	SetTrieValue(g_hAmmoList, "weapon_mp9", 120);
	SetTrieValue(g_hAmmoList, "weapon_bizon", 120);
	SetTrieValue(g_hAmmoList, "weapon_sawedoff", 32);
	SetTrieValue(g_hAmmoList, "weapon_mag7", 32);
	SetTrieValue(g_hAmmoList, "weapon_negev", 200);
	SetTrieValue(g_hAmmoList, "weapon_m4a1_silencer", 40);
	SetTrieValue(g_hAmmoList, "weapon_usp_silencer", 24);
	SetTrieValue(g_hAmmoList, "weapon_cz75a", 12);

	CloseHandle(hConfig);
}